/*
 * Copyright (C) 2021 Zorin OS Technologies Ltd.
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>

#include "plugin.h"
#include "slot.h"

extern "C"
{
#include <libxfce4util/libxfce4util.h>
#include <libxfce4ui/libxfce4ui.h>
}

using namespace Xfce4ZorinAppGridLitePlugin;

//-----------------------------------------------------------------------------

extern "C" void xfce4_zorinappgridlite_plugin_construct(XfcePanelPlugin* plugin)
{
	new Plugin(plugin);
}

static void xfce4_zorinappgridlite_plugin_free(XfcePanelPlugin*, Plugin* xfce4_zorinappgridlite_plugin)
{
	delete xfce4_zorinappgridlite_plugin;
}

static void widget_add_css(GtkWidget* widget, const gchar* css)
{
	GtkCssProvider* provider = gtk_css_provider_new();
	gtk_css_provider_load_from_data(provider, css, -1, NULL);
	gtk_style_context_add_provider(gtk_widget_get_style_context(widget),
			GTK_STYLE_PROVIDER(provider),
			GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	g_object_unref(provider);
}

//-----------------------------------------------------------------------------

Plugin::Plugin(XfcePanelPlugin* plugin) :
	m_plugin(plugin)
{
	// Create button
	m_button = xfce_panel_create_button();
	gtk_widget_set_name(m_button, "zorinappgridlite-button");
#if !LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
	widget_add_css(m_button, ".xfce4-panel button { padding: 1px; }");
#endif
	g_signal_connect_slot(m_button, "clicked", &Plugin::button_clicked, this);
	gtk_widget_show(m_button);

	m_button_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2));
	gtk_container_add(GTK_CONTAINER(m_button), GTK_WIDGET(m_button_box));
	gtk_container_set_border_width(GTK_CONTAINER(m_button_box), 0);
	gtk_widget_show(GTK_WIDGET(m_button_box));

	m_button_icon = GTK_IMAGE(gtk_image_new());
	gtk_image_set_from_icon_name (m_button_icon,
                                  "xfce4-zorinappgridlite-plugin-symbolic", GTK_ICON_SIZE_MENU);
	gtk_box_pack_start(m_button_box, GTK_WIDGET(m_button_icon), true, false, 0);
	gtk_widget_show(GTK_WIDGET(m_button_icon));
	gtk_widget_set_sensitive(GTK_WIDGET(m_button_icon), false);

	// Add plugin to panel
	gtk_container_add(GTK_CONTAINER(plugin), m_button);
	xfce_panel_plugin_add_action_widget(plugin, m_button);

	// Connect plugin signals to functions
	g_signal_connect(plugin, "free-data", G_CALLBACK(xfce4_zorinappgridlite_plugin_free), this);
	g_signal_connect_slot(plugin, "mode-changed", &Plugin::mode_changed, this);
	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "about", &Plugin::show_about, this);
	g_signal_connect_slot(plugin, "size-changed", &Plugin::size_changed, this);

	xfce_panel_plugin_menu_show_about(plugin);

	mode_changed(m_plugin, xfce_panel_plugin_get_mode(m_plugin));

	g_signal_connect_slot<GtkWidget*>(plugin, "style-updated", &Plugin::update_size, this);
	g_signal_connect_slot<GtkWidget*,GdkScreen*>(m_button, "screen-changed", &Plugin::update_size, this);
}

//-----------------------------------------------------------------------------

Plugin::~Plugin()
{
	gtk_widget_destroy(m_button);
}

//-----------------------------------------------------------------------------

void Plugin::set_loaded(bool loaded)
{
	gtk_widget_set_sensitive(GTK_WIDGET(m_button_icon), loaded);
}

//-----------------------------------------------------------------------------

void Plugin::button_clicked(GtkButton* button)
{
	system("zorin-appgrid-lite");
}

//-----------------------------------------------------------------------------

void Plugin::mode_changed(XfcePanelPlugin*, XfcePanelPluginMode)
{
	update_size();
}

//-----------------------------------------------------------------------------

void Plugin::show_about()
{
	const gchar* authors[] = {
		"Kyrill Zorin <zorink@zoringroup.com>",
		"Artyom Zorin <azorin@zoringroup.com>",
		NULL };

	gtk_show_about_dialog
		(NULL,
		"authors", authors,
		"comments", ("A simple, lightweight, and powerful application launcher"),
		"copyright", ("Copyright \302\251 2021 Zorin OS Technologies Ltd."),
		"license", XFCE_LICENSE_GPL,
		"logo-icon-name", "xfce4-zorinappgridlite-plugin",
		"program-name", "Zorin AppGrid Lite",
		"version", PACKAGE_VERSION,
		NULL);
}

//-----------------------------------------------------------------------------

gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
{
	GtkOrientation panel_orientation = xfce_panel_plugin_get_orientation(m_plugin);
	GtkOrientation orientation = panel_orientation;

	// Make icon expand to fill button
	gtk_box_set_child_packing(GTK_BOX(m_button_box), GTK_WIDGET(m_button_icon),
			true,
			true,
			0, GTK_PACK_START);

	// Resize icon
#if LIBXFCE4PANEL_CHECK_VERSION(4,13,0)
	gint icon_size;
	if (size <= 19)
		icon_size = 12;
	else if (size <= 27)
		icon_size = 16;
	else if (size <= 35)
		icon_size = 24;
	else if (size <= 41)
		icon_size = 32;
	else
		icon_size = size - 4;
#else
	GtkBorder padding, border;
	GtkStyleContext* context = gtk_widget_get_style_context(m_button);
	GtkStateFlags flags = gtk_widget_get_state_flags(m_button);
	gtk_style_context_get_padding(context, flags, &padding);
	gtk_style_context_get_border(context, flags, &border);
	gint xthickness = padding.left + padding.right + border.left + border.right;
	gint ythickness = padding.top + padding.bottom + border.top + border.bottom;
	gint icon_size = size - 2 * std::max(xthickness, ythickness);
#endif
	gtk_image_set_pixel_size(m_button_icon, icon_size);

	gtk_widget_set_size_request(m_button, size, size);

	xfce_panel_plugin_set_small(m_plugin, false);

	gtk_orientable_set_orientation(GTK_ORIENTABLE(m_button_box), orientation);

	return true;
}

//-----------------------------------------------------------------------------

void Plugin::update_size()
{
	size_changed(m_plugin, xfce_panel_plugin_get_size(m_plugin));
}

//-----------------------------------------------------------------------------
